#include <cmath>
#include "time.h"
#include <boost/gil/image.hpp>
#include <boost/gil/extension/io/png_io.hpp>

using namespace boost::gil;

// example of a functor (just class with operator())
class MyFunctor {
private:

public:
	gray8_pixel_t operator()(int x, int y) {
		gray8_pixel_t tmp((x + y + std::rand() % 50) % 256);
		return tmp;
	}

};

gray8_pixel_t constGray(int x, int y) {
	return gray8_pixel_t(200);
}

// example of a template function using a functor or function
// as template parameter
template<typename FUNCTION>
gray8_pixel_t myFunction(int x, int y, FUNCTION func) {
	return func(x, y);
}


int main() {
	// initialize random number generator with "random" value
	srand ( time(NULL) );

	typedef gray8_image_t image_t;

	// image size
	image_t::point_t dims(1000, 1000);

	// allocate image and create view
	image_t myImage(dims);
	image_t::view_t myView = view(myImage);

	MyFunctor myFunctor;


	for(int y = 0; y < dims.y; ++y) {
		for(int x = 0; x < dims.x; ++x) {
			myView(x, y) = myFunction<MyFunctor>(x, y, myFunctor);
			//myView(x, y) = myFunction<gray8_pixel_t(*)(int, int)>(x, y, constGray);
		}
	}

	// example of simple derivation filter in x-direction
	//for(int y = 0; y < dims.y; ++y) {
		//for(int x = 0; x < dims.x - 1; ++x) {
			//myView(x, y) = (myView(x + 1, y) - myView(x, y) + 255) / 2;
		//}
	//}

	// write myView to file test.png
	png_write_view("test.png", myView);

	return 0;
}
